package swingtables.gui;

import java.sql.SQLException;
import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import swingtables.logic.DbMovie;
import swingtables.logic.MoviesDatasource;
import swingtables.logic.DbGenre;

/**
 *
 * @author trust56
 */
class MoviesTableModel extends AbstractTableModel implements TableModel {

    private DbMovie getMovieAtRow(int rowIndex) throws SQLException {
        return MoviesDatasource.getInstance().getMovies().get(rowIndex);
    }

    @Override
    public int getRowCount() {
        try {
            return MoviesDatasource.getInstance().getMovies().size();
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
            return 0;
        }
    }

    @Override
    public int getColumnCount() {
        return 4;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        DbMovie movie;
        try {
            movie = getMovieAtRow(rowIndex);
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
            return "Error";
        }
        switch (columnIndex) {
            case 0:
                return movie.getTitle();
            case 1:
                return movie.getYearOfProduction();
            case 2:
                return movie.isColor();
            case 3:
                return movie.getGenre();
            default:
                return null;
        }
    }

    @Override
    public String getColumnName(int column) {
        return new String[]{"Title", "Year", "Color", "Genre"}[column];
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return String.class;
            case 1:
                return Integer.class;
            case 2:
                return Boolean.class;
            case 3:
                return DbGenre.class;
            default:
                return null;
        }
    }

    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        DbMovie movie;
        try {
            movie = getMovieAtRow(rowIndex);
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
            return;
        }
        switch (columnIndex) {
            case 0:
                movie.setTitle(aValue.toString());
                break;
            case 1:
                try {
                    movie.setYearOfProduction(Integer.valueOf(aValue.toString()));
                } catch (IllegalArgumentException ex) {
                    JOptionPane.showMessageDialog(null, "Movie production year is too early", "Error", JOptionPane.ERROR_MESSAGE);
                }
                break;
            case 2:
                movie.setColor(Boolean.valueOf(aValue.toString()));
                break;
            case 3:
                movie.setGenre((DbGenre) aValue);
                break;
        }
        try {
            MoviesDatasource.getInstance().updateMovie(movie, rowIndex);
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
        }
    }

    public void addNewMovie() {
        int index;
        try {
            index = MoviesDatasource.getInstance().addMovie();
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
            return;
        }
        fireTableRowsInserted(index, index);
    }

    public void deleteMovie(int rowIndex) {
        try {
            MoviesDatasource.getInstance().deleteMovie(rowIndex);
        } catch (SQLException ex) {
            SwingTablesFrame.displayError(ex);
        }
        fireTableRowsDeleted(rowIndex, rowIndex);
    }
}
