package swingtables.logic;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class MoviesDatasource {

    private String url, user, password;

    private MoviesDatasource() {
        url = "jdbc:derby://localhost:1527/movies";
        user = "root";
        password = "root";
    }
    private static MoviesDatasource moviesDatasourceInstance = null;

    public static MoviesDatasource getInstance() {
        if (moviesDatasourceInstance == null) {
            moviesDatasourceInstance = new MoviesDatasource();
        }
        return moviesDatasourceInstance;
    }

    Connection getConnection() throws SQLException {
        return DriverManager.getConnection(url, user, password);
    }

    public List<DbMovie> getMovies() throws SQLException {
        List<DbMovie> movies = new ArrayList<>();
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(); ResultSet rs = stmt.executeQuery("SELECT * FROM MOVIE")) {
            while (rs.next()) {
                DbMovie movie = new DbMovie(rs);
                movies.add(movie);
            }
        }
        return movies;
    }

    public List<DbGenre> getGenres() throws SQLException {
        List<DbGenre> genres = new ArrayList<>();
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(); ResultSet rs = stmt.executeQuery("SELECT * FROM GENRE")) {
            while (rs.next()) {
                DbGenre genre = new DbGenre(rs);
                genres.add(genre);
            }
        }
        return genres;
    }

    public int obtainNewId() throws SQLException {
        int id;
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_UPDATABLE); ResultSet rs = stmt.executeQuery("SELECT VALUUE FROM SEQ")) {
            rs.next();
            id = rs.getInt("VALUUE") + 1;
            rs.updateInt("VALUUE", id);
            rs.updateRow();
        }
        return id;
    }

    public int addMovie() throws SQLException {
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE); ResultSet rs = stmt.executeQuery("SELECT * FROM MOVIE")) {
            rs.moveToInsertRow();
            DbMovie.addMovie(rs);
            rs.insertRow();
        }
        return 1;
    }

    public void deleteMovie(int index) throws SQLException {
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE); ResultSet rs = stmt.executeQuery("SELECT * FROM MOVIE")) {
            rs.absolute(index + 1);
            rs.deleteRow();
        }
    }

    public void updateMovie(DbMovie movie, int index) throws SQLException {
        try (Connection connection = getConnection(); Statement stmt = connection.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE); ResultSet rs = stmt.executeQuery("SELECT * FROM MOVIE")) {
            rs.absolute(index + 1);
            movie.update(rs);
            rs.updateRow();
        }
    }
}
